-- coffee machine state machine
beginStateMachine()
	
	onEnter(function(msg)
		local coffeeMachine = getStateObjectFromID(msg.sender);
		storeStateObject("coffeeMachine", coffeeMachine);
		
		
		if (coffeeMachine) then
			-- coffeeMachine does exist
			local actionPointName = retrieveData("actionPointName");
			local actionPoint = getParent().getFreeActionPoint(coffeeMachine, actionPointName);

			if exitIfWrongPosition(getParent(), coffeeMachine, actionPointName) then return end;	

			if (not actionPoint) then
				-- action point is locked
				getParent().setEmoticon(EMOTICON_CANNOT, EMOTICON_DELAY);
				sendMsg("emoThink", getParent().walkSO);
				exitStateMachine();
			else
				getParent().lockActionPoints(coffeeMachine);
			end
		else
			-- basket does not exist anymore
			getParent().setEmoticon(EMOTICON_CANNOT, EMOTICON_DELAY);
			sendMsg("emoThink", getParent().walkSO);
			exitStateMachine();
		end
		
		freeHands(getParent());
				
	end )
	
	onExit(function(msg)
		unlockAll("coffeeMachine");		
		
		getParent().stopAllActivities();	
	end )
	
	-----------------------------------------------------------------------------------------	
	
	state("cookCoffee")
	
		onEnter(function(msg)
		
			local coffeeMachine = retrieveStateObject("coffeeMachine");
						
			startAnimation("cookCoffee");
			coffeeMachine.startAnimation("cookCoffee_coffeemachine");
			
			sendDelayedMsgThis("grabSpoon", 1400);
			sendDelayedMsgThis("dropSpoon", 5700);
		end )
		
		
		onMsg("grabSpoon", function(msg)
			local coffeeMachine = retrieveStateObject("coffeeMachine");
			local coffeeSpoon = coffeeMachine.createGameObject("CoffeeSpoon");
			storeStateObject("coffeeSpoon", coffeeSpoon);
			getParent().attachLeftObjectHolder(coffeeSpoon);
		end )			

		onMsg("dropSpoon", function(msg)
			local coffeeSpoon = getParent().getLeftHeldObject();
			if (coffeeSpoon) then
				getParent().detachLeftObjectHolder();
				coffeeSpoon.deleteGameObject();
			end
		end )			


		onMsg("end", function(msg)
		
			local coffeeMachine = retrieveStateObject("coffeeMachine");
			
			if testCancel() then
				exitAndGoAway();
			else
			
				local damp = coffeeMachine.findChildGO("damp");
				--damp.setEmitRate(0.0);
				coffeeMachine.setChildEnable("damp", true);			
				sendDelayedMsg("coolDown", coffeeMachine, COFFEE_COOLDOWN_TIME);
			
				--setState("pourCoffee");
				
				getParent().pushCommand("pm_drinkCoffee", "use", coffeeMachine, "pourCoffee");
				exitStateMachine();
				
			end
		end )	
	
	-----------------------------------------------------------------------------------------	
	
	state("pourCoffee")
	
		onEnter(function(msg)
			
			local coffeeMachine = retrieveStateObject("coffeeMachine");
			
			startAnimation("pourCoffee");
			coffeeMachine.startAnimation("pourCoffee_coffeemachine");
			
			sendDelayedMsgThis("grabCup", 2100);
			sendDelayedMsgThis("grabPot", 1800);
			sendDelayedMsgThis("dropPot", 5400);
			
			sendDelayedMsgThis("testCancel", CANCEL_POLLING_INTERVAL);
			
		end )
		
		onExit(function(msg)
			local coffeePot = getParent().getRightHeldObject();
			if (coffeePot) then
				getParent().detachRightObjectHolder();
				coffeePot.deleteGameObject();
			end
			
			local coffeeMachine = retrieveStateObject("coffeeMachine");
			getParent().unlockActionPoints(coffeeMachine);

		end )

		onMsg("grabCup", function(msg)
			local coffeeCup = getParent().loadGameObject("StandardGO", "coffeeCup");
			--storeStateObject("coffeeCup", coffeeCup);
			getParent().attachLeftObjectHolder(coffeeCup);
		end )			
				
		onMsg("grabPot", function(msg)
			local coffeeMachine = retrieveStateObject("coffeeMachine");
			local coffeePot = coffeeMachine.createGameObject("Coffeepot");
			storeStateObject("coffeePot", coffeePot);
			getParent().attachRightObjectHolder(coffeePot);
		end )			

		onMsg("dropPot", function(msg)
			local coffeePot = getParent().getRightHeldObject();
			if (coffeePot) then
				getParent().detachRightObjectHolder();
				coffeePot.deleteGameObject();
			end
			
			-- enable cup damp
			local coffeeCup = getParent().getLeftHeldObject();
			local damp = coffeeCup.findChildGO("damp");
			damp.setEmitRate(0.5);
			coffeeCup.setChildEnable("damp", true);
			
		end )			

		onMsg("complete", function(msg)
		
			local actionPoint = getParent().findPointNear(1.0, 5.0);
			if (actionPoint) then
			
				if (walkToPointImmediate(actionPoint)) then
					local wsoContext = StateMachineContext();
					getParent().handSO.setPose(genderizeReal(getParent(), "leftHandHoldCoffee"));
					enterStateMachine("subwalk.walk");
				else
					print("no path found");
					sendMsgThis("canceled");
				end	
			else
				print("no near free space found");
				setState("drinkCoffee");
			end
		
		
		end )
		
		onReturn(function(msg)
			-- back from subwalk
			print("onReturn pour");
			setState("drinkCoffee");
		end )
				

		onMsg("testCancel", function(msg)
			if testCancel() then
				sendMsgThis("canceled");
			else
				sendDelayedMsgThis("testCancel", CANCEL_POLLING_INTERVAL);
			end
		end )			


		onMsg("canceled", function(msg)
			local coffeeCup = getParent().getLeftHeldObject();
			if (coffeeCup) then
				print("deleting coffeeCup")
				getParent().detachLeftObjectHolder();
				coffeeCup.deleteGameObject();
			end
			exitAndGoAway();
		end)


		onMsg("end", function(msg)
			if testCancel() then
				sendMsgThis("canceled");
			else
				sendMsgThis("complete");
			end
		end )	
	
	-----------------------------------------------------------------------------------------	
	
	state("drinkCoffee")
	
		onEnter(function(msg)
			
			if testCancel() then
				exitStateMachine();
			else
				local coffeeCup = getParent().getLeftHeldObject();
				local coffeeMachine = retrieveStateObject("coffeeMachine");
				local drinkCoffee = getParent().startActivity("drinkCoffee", coffeeMachine);
				local length = getActivityLength(drinkCoffee);
				
				--flipPoseDirection(); -- animation ist wrong oriented
				
				sendDelayedMsgThis("complete", length);
				--sendDelayedMsgThis("testCancel", CANCEL_POLLING_INTERVAL);
				sendMsgThis("startLoop");
				
				coffeeCup.buy(); -- pay money for food
			end
			
		end )
		
		onExit(function(msg)
			local coffeeCup = getParent().getLeftHeldObject();
			if (coffeeCup) then
				print("deleting coffeeCup")
				getParent().stopAllActivities(coffeeCup);
				getParent().detachLeftObjectHolder();
				coffeeCup.deleteGameObject();
			end
			
			freeHands(getParent());
			
--			createTrash(0.1);
			
			--flipPoseDirection(); -- animation ist wrong oriented
		end )
		
		
		onMsg("startLoop", function(msg)
			getParent().handSO.stopAnimation(0.0);
			startAnimation(genderizeReal(getParent(), "drinkCoffee"), false, 1.0, 0.0);
			sendDelayedMsgThis("drinkSound", 1000);
		end )
		
		onMsg("drinkSound", function(msg)
			getParent().playSound(genderize(getParent(), "drink"));
		end )
					
		onMsg("testCancel", function(msg)
			if testCancel() or (not this.getParent().getCurrentActivityGain()) then
				exitStateMachine();
			else
				sendDelayedMsgThis("testCancel", CANCEL_POLLING_INTERVAL);
			end
		end )

		onMsg("complete", function(msg)
			exitStateMachine()
		end )
		
		onMsg("end", function(msg)
			if testCancel() then
				exitStateMachine();
			else
				sendDelayedMsgThis("startLoop", math.random(2000, 5000));
			end
		end )	

		

endStateMachine()
